@@ -23,7 +23,7 @@ from collections import OrderedDict
 from collections.abc import KeysView
 import yaml
 import numpy as np
-from distutils.version import StrictVersion
+from packaging.version import Version
 from neural_compressor.adaptor.adaptor import adaptor_registry, Adaptor
 from neural_compressor.adaptor.query import QueryBackendCapability
 from neural_compressor.utils.utility import LazyImport, dump_elapsed_time, \
@@ -36,8 +36,9 @@ import math
 onnx = LazyImport("onnx")
 ort = LazyImport("onnxruntime")
 ort_ext = LazyImport("onnxruntime_extensions")
-ONNXRT152_VERSION = StrictVersion("1.5.2")
-ONNXRT170_VERSION = StrictVersion("1.7.0")
+ONNXRT152_VERSION = Version("1.5.2")
+ONNXRT170_VERSION = Version("1.7.0")
+ONNXRT112_VERSION = Version("1.12.0")
 
 logger = logging.getLogger()
 
@@ -94,13 +95,13 @@ class ONNXRTAdaptor(Adaptor):
         """
         assert q_func is None, "quantization aware training has not been supported on ONNXRUNTIME"
         model = self.pre_optimized_model if self.pre_optimized_model else model
-        ort_version = StrictVersion(ort.__version__)
+        ort_version = Version(ort.__version__)
         if ort_version < ONNXRT152_VERSION: # pragma: no cover
             logger.warning("Quantize input needs onnxruntime 1.5.2 or newer.")
             return model
         if model.model.opset_import[0].version < 11: # pragma: no cover
             logger.warning("Quantize input needs model opset 11 or newer.")
-        from onnxruntime.quantization.quant_utils import QuantizationMode
+        from neural_compressor.adaptor.ox_utils.util import QuantizationMode
         if self.backend in ["qlinearops", "qoperator"]:
             backend = QuantizationMode.QLinearOps
             if self.backend == "qlinearops":
@@ -136,7 +137,11 @@ class ONNXRTAdaptor(Adaptor):
                     data_loader.batch(calib_batch_size)
                     quantize_params = self._get_quantize_params(tmp_model.model, data_loader, \
                                                                 quantize_config, tmp_iterations)
-                except Exception:  # pragma: no cover
+                except Exception as e:  # pragma: no cover
+                    if 'Got invalid dimensions for input' in str(e):
+                        logger.warning("Please set sampling_size to a multiple of {}".format(
+                            str(e).partition('Expected: ')[2].partition('\n')[0]))
+                        exit(0)
                     logger.warning(
                         "Fail to forward with batch size={}, set to {} now.".
                         format(batch_size, 1))
@@ -211,14 +216,14 @@ class ONNXRTAdaptor(Adaptor):
         """
         self._pre_optimize(model)
         model = self.pre_optimized_model
-        ort_version = StrictVersion(ort.__version__)
+        ort_version = Version(ort.__version__)
         if ort_version < ONNXRT152_VERSION: # pragma: no cover
             logger.warning("Quantize input needs onnxruntime 1.5.2 or newer.")
             return model
         if model.model.opset_import[0].version < 11: # pragma: no cover
             logger.warning("Quantize input needs model opset 11 or newer.")
 
-        from onnxruntime.quantization.quant_utils import QuantizationMode
+        from neural_compressor.adaptor.ox_utils.util import QuantizationMode
         if self.backend in ["qlinearops", "qoperator"]:
             backend = QuantizationMode.QLinearOps
         elif self.backend == "qdq":
@@ -269,59 +274,29 @@ class ONNXRTAdaptor(Adaptor):
         for precision in self.query_handler.get_precisions():
             if precision != 'fp32':
                 fp32_op_list += self.query_handler.get_op_types_by_precision(precision=precision)
+        qdq_ops = ["QuantizeLinear", "DequantizeLinear", "DynamicQuantizeLinear"]
         res = {}
         for op_type in fp32_op_list:
             res[op_type] = {'INT8':0, 'BF16': 0, 'FP16': 0, 'FP32':0}
-        for op_type in ["QuantizeLinear", "DequantizeLinear", "DynamicQuantizeLinear"]:
+        for op_type in qdq_ops:
             res[op_type] = {'INT8':0, 'BF16': 0, 'FP16': 0, 'FP32':0}
 
-
-        if self.backend in ["qlinearops", "qdq", "qoperator"] :
-            int8_op_list = ["QLinearConv", "QLinearMatMul", "QAttention",
-                            "QLinearMul", "QLinearRelu", "QLinearClip",
-                            "QLinearLeakyRelu", "QLinearSigmoid", "MaxPool","Squeeze",
-                            "EmbedLayerNormalization", "QLinearGlobalAveragePool", 
-                            "QLinearAdd", "Pad", "Split", "Gather", "Reshape", "Concat",
-                            "QuantizeLinear", "DequantizeLinear", "QLinearAveragePool",
-                            "Unsqueeze", "Transpose"
-            ]
-        else:
-            int8_op_list = ["ConvInteger", "MatMulInteger", "QAttention",
-                            "DynamicQuantizeLSTM", "Gather", "EmbedLayerNormalization",
-                            "DynamicQuantizeLinear"
-            ]
-
         for node in model.model.graph.node:
-            possible_int8_res = [name for name in int8_op_list if node.op_type.find(name) != -1]
- 
-            if any(possible_int8_res):
+            if node.name.endswith('_quant'):
                 if self.backend in ["qlinearops", "qdq", "qoperator"]:
-                    if node.op_type == "QuantizeLinear" or node.op_type == "DequantizeLinear" \
-                            or node.op_type == "DynamicQuantizeLinear":
-                        origin_op_type = node.op_type
-                    else:
-                        origin_op_type = possible_int8_res[0].split('QLinear')[-1]
+                    origin_op_type = node.op_type.split('QLinear')[-1]
                 else:
-                    origin_op_type = possible_int8_res[0].split('Integer')[0]
+                    origin_op_type = node.op_type.split('Integer')[0]
 
-                if node.op_type in ["Pad", "Split", "Gather", "Concat", "Reshape", "Unsqueeze", 
-                    "Squeeze", "Transpose"]:
-                    if any([output.endswith('_quantized') for output in node.output]) or \
-                        any(['_DequantizeLinear' in inp for inp in node.input]):
-                        origin_op_type = node.op_type
-                    else:
-                        if node.op_type in res:
-                            res[node.op_type]['FP32'] += 1
-                        continue
-
-                if origin_op_type == "QAttention":
-                    origin_op_type = "Attention"
-                if origin_op_type == "DynamicQuantizeLSTM":
+                if origin_op_type in ["QAttention", "QGemm"]:
+                    origin_op_type = origin_op_type[1:]
+                elif origin_op_type == "DynamicQuantizeLSTM":
                     origin_op_type = "LSTM"
+                elif origin_op_type == "QEmbedLayerNormalization":
+                    origin_op_type = "EmbedLayerNormalization"
                 res[origin_op_type]['INT8'] += 1
 
-            elif node.op_type in fp32_op_list and \
-                any(['_DequantizeLinear' in inp for inp in node.input]):
+            elif node.op_type in qdq_ops:
                 res[node.op_type]['INT8'] += 1
 
             elif node.op_type in fp32_op_list and node.name in self.quantize_config:
@@ -330,6 +305,9 @@ class ONNXRTAdaptor(Adaptor):
                 else:
                     res[node.op_type][self.quantize_config[node.name].upper()] += 1
 
+            elif node.op_type in res:
+                res[node.op_type]['FP32'] += 1
+
         output_data = [[op_type, sum(res[op_type].values()), res[op_type]['INT8'],
             res[op_type]['BF16'], res[op_type]['FP16'], res[op_type]['FP32']] for \
             op_type in res.keys()]
@@ -416,6 +394,13 @@ class ONNXRTAdaptor(Adaptor):
                     self.quantize_config[node_name]['weight']['scheme'],
                     scale_value,
                     zo_value)
+            elif (Version(ort.__version__) >= ONNXRT112_VERSION and \
+                model.model.opset_import[0].version < 13) and \
+                len(scale_tensor.dims) in [1, 2]:
+                logger.warning("Skip setting per-channel quantized tensor {}, please " \
+                    "use onnxruntime < 1.12.0 or upgrade model opset version to 13 or " \
+                    "higher".format(tensor_name))
+                return model
             else:
                 new_tensor_value = quantize_data_per_channel(
                     tensor_value,
@@ -471,7 +456,7 @@ class ONNXRTAdaptor(Adaptor):
         self.pre_optimized_model = model
 
     def _revert_fusedconv(self, model):
-        from onnxruntime.quantization.quant_utils import attribute_to_kwarg
+        from neural_compressor.adaptor.ox_utils.util import attribute_to_kwarg
         from onnx import onnx_pb as onnx_proto
         new_nodes = []
         remove_nodes = []
@@ -813,7 +798,7 @@ class ONNXRTAdaptor(Adaptor):
 
     def diagnosis_helper(self, fp32_model, int8_model, tune_cfg=None, save_path=None):
         from neural_compressor.utils.utility import dump_data_to_local
-        from neural_compressor.model.onnx_model import find_by_name
+        from neural_compressor.adaptor.ox_utils.util import find_by_name
         if self.backend in ["qlinearops", "qoperator"]:
             supported_optype = ['Conv', 'MatMul', 'Concat', 'Attention', 'FusedConv',
                 'Add', 'Mul', 'LeakyRelu', 'Sigmoid', 'GlobalAveragePool', 'AveragePool']
@@ -938,15 +923,37 @@ class ONNXRTQuery(QueryBackendCapability):
         Returns:
             [dictionary]: the content for specific version.
         """
-        default_config = None
+        from functools import cmp_to_key
+        config = None
+
+        def _compare(version1, version2):
+            if Version(version1[0]) == Version(version2[0]):
+                return 0
+            elif Version(version1[0]) < Version(version2[0]):
+                return -1
+            else:
+                return 1
+
+        extended_cfgs = []
         for sub_data in data:
-            if self.version in sub_data['version']['name']:
-                return sub_data
-
             if 'default' in sub_data['version']['name']:
-                default_config = sub_data
+                assert config == None, "Only one default config " \
+                    "is allowed in framework yaml file."
+                config = sub_data
+            versions = sub_data['version']['name'] if \
+                isinstance(sub_data['version']['name'], list) else \
+                [sub_data['version']['name']]
+            for version in versions:
+                if version != 'default':
+                    extended_cfgs.append((version, sub_data))
 
-        return default_config
+        extended_cfgs = sorted(extended_cfgs, key=cmp_to_key(_compare), reverse=True)
+        for k, v in extended_cfgs:
+            if Version(self.version) >= Version(k):
+                config = v
+                break
+
+        return config
 
     def get_version(self): # pragma: no cover
         """Get the current backend version infomation.
