@@ -18,7 +18,7 @@
 
 import onnx
 from .base_operator import QuantOperatorBase
-from onnxruntime.quantization.quant_utils import attribute_to_kwarg, ms_domain, \
+from neural_compressor.adaptor.ox_utils.util import attribute_to_kwarg, ms_domain, \
                                                  QuantizedValueType
 from neural_compressor.adaptor.ox_utils.util import QuantizedValue                                                 
 class QGlobalAveragePool(QuantOperatorBase):
@@ -47,7 +47,7 @@ class QGlobalAveragePool(QuantOperatorBase):
             "QLinear" + node.op_type,
             inputs,
             child.output,
-            node.name, **kwargs)
+            node.name + '_quant', **kwargs)
         self.quantizer.new_nodes += [qnode]
         self.quantizer.remove_nodes.append(child)
         self.quantizer.remove_nodes.append(parent)
