@@ -27,25 +27,22 @@ import copy
 from onnx import onnx_pb as onnx_proto
 from onnx import TensorProto
 from onnx import shape_inference
-from onnxruntime.quantization.quant_utils import QuantizedValueType
-from onnxruntime.quantization.quant_utils import find_by_name, get_elem_index, get_mul_node, \
-                                generate_identified_filename, attribute_to_kwarg, type_to_name
-from onnxruntime.quantization.quant_utils import __producer__, __version__, onnx_domain
 from onnxruntime import SessionOptions, InferenceSession, GraphOptimizationLevel
-from onnxruntime.quantization.quant_utils import QuantizationMode
 
 from neural_compressor.adaptor.ox_utils.registry import CreateQDQQuantizer, \
     CreateOpConverter, CreateCaster
 from neural_compressor.adaptor.ox_utils.util import QuantizedValue, QuantizedInitializer, \
-    quantize_data_with_scale_zero, quantize_data, dtype_mapping, support_pair, ValueInfo, \
-    _get_qrange_for_qType, convert_np_to_float16, cast_tensor, make_quant_node, make_dquant_node
+    _get_qrange_for_qType, cast_tensor, make_quant_node, make_dquant_node
+from neural_compressor.adaptor.ox_utils.util import QuantizedValueType
+from neural_compressor.adaptor.ox_utils.util import find_by_name, dtype_to_name
+from neural_compressor.adaptor.ox_utils.util import __producer__, __version__
+from neural_compressor.adaptor.ox_utils.util import quantize_data, dtype_mapping, support_pair, ValueInfo
 from neural_compressor import options
 from neural_compressor.utils.utility import CpuInfo
 from neural_compressor.model.onnx_model import ONNXModel
 
 logger = logging.getLogger()
 
-
 class Quantizer:
     def __init__(self, model, q_config, mode, static, quantization_params,
                  op_types_to_quantize, fallback_list=['fp32']):
@@ -110,7 +107,7 @@ class Quantizer:
             opset_version = 11
 
         if opset_version < 13 and self.mode == 'qdq':
-            logger.warning("Per-Channel support with QDQ format requires onnx opset >= 13," \
+            logger.warning("Per-channel support with QDQ format requires opset version >= 13," \
                 " use per-tensor granularity instead")
         return opset_version
 
@@ -171,6 +168,11 @@ class Quantizer:
                             self.replace_input.append([self.model.get_children(child)[0],
                                                 child.output[0], node.input[0]])
                     self.remove_nodes.append(node)
+            self.model.remove_nodes(self.remove_nodes)
+            self.model.graph().node.extend(self.new_nodes)
+            for node, old_input_name, new_input_name in self.replace_input:
+                self.model.replace_node_input(node, old_input_name, new_input_name)
+            self.model.update()
         elif self.mode != 'qdq' or not self.dedicated_qdq_pair:
             target_type = ['QuantizeLinear', 'DequantizeLinear']
             for op_type in target_type:
@@ -192,11 +194,11 @@ class Quantizer:
                             self.replace_input.append([self.model.get_children(dq_nodes[i])[0],
                                                        dq_nodes[i].output[0], 
                                                        dq_nodes[idx].output[0]])
-        self.model.remove_nodes(self.remove_nodes)
-        self.model.graph().node.extend(self.new_nodes)
-        for node, old_input_name, new_input_name in self.replace_input:
-            self.model.replace_node_input(node, old_input_name, new_input_name)
-        self.model.update()
+                self.model.remove_nodes(self.remove_nodes)
+                self.model.graph().node.extend(self.new_nodes)
+                for node, old_input_name, new_input_name in self.replace_input:
+                    self.model.replace_node_input(node, old_input_name, new_input_name)
+                self.model.update()
 
     def should_cast(self, node):
         if node.name in self.config and self.config[node.name] != 'fp32': # pragma: no cover
@@ -271,9 +273,10 @@ class Quantizer:
  
                         self.remove_nodes.append(match_nodes[1])
                         if all([i.op_type in ['QuantizeLinear', 'DequantizeLinear'] \
-                            for i in self.model.get_children(match_nodes[0])]):
+                            for i in self.model.get_children(match_nodes[0])]) and \
+                            match_nodes[0].output[0] not in self.model.output():
                             self.remove_nodes.append(match_nodes[0])
-                else:
+                else: # pragma: no cover
                     parent = self.model.get_parents(match_nodes[0])[0]
                     children = self.model.get_children(match_nodes[1])
                     input_dtype = '1' # float32
@@ -328,9 +331,12 @@ class Quantizer:
             if initializer is not None:
                 if initializer.data_type != onnx_proto.TensorProto.FLOAT: 
                     continue
-                cast_tensor(initializer, cfg, min_positive_val, max_finite_val)
-                self.new_value_info[tensor_name] = ValueInfo(tensor_name,
-                                                         TensorProto.FLOAT, dtype_mapping[cfg])
+                new_tensor = cast_tensor(initializer, cfg)
+                if new_tensor:
+                    self.model.remove_initializer(initializer)
+                    self.model.add_initializer(new_tensor)
+                    self.new_value_info[tensor_name] = ValueInfo(tensor_name,
+                                                             TensorProto.FLOAT, dtype_mapping[cfg])
             else:
                 if tensor_name in self.value_infos and \
                     self.value_infos[tensor_name].type.HasField('tensor_type') and \
@@ -557,16 +563,27 @@ class Quantizer:
             self._dynamic_quantize_bias(input_name, weight_name + '_scale', bias_name,
                 bias_name + "_quantized")
         else:
-            _, quant_value = self.quantize_bias(bias_name, input_name, weight_name)
+            beta = 1.0
+            if node.op_type in ['Gemm']:
+                beta_attribute = [attr for attr in node.attribute if attr.name == "beta"]
+                if len(beta_attribute):
+                    beta = onnx.helper.get_attribute_value(beta_attribute[0])
+            _, quant_value = self.quantize_bias(bias_name, input_name, weight_name, beta)
             self.model.remove_initializer(find_by_name(bias_name, self.model.initializer()))
             inputs = [quant_value.q_name, quant_value.scale_name, quant_value.zp_name]
-            dequant_node = onnx.helper.make_node("DequantizeLinear", inputs, 
-                [bias_name + '_dequantized'], bias_name + '_DequantizeLinear')
+            axis = None
+            if find_by_name(weight_name + '_DequantizeLinear', self.new_nodes):
+                dq_node = find_by_name(weight_name + '_DequantizeLinear', self.new_nodes)
+                if dq_node.op_type == 'DequantizeLinear' and \
+                    find_by_name('axis', dq_node.attribute):
+                    axis = find_by_name('axis', dq_node.attribute).i
+            dequant_node = make_dquant_node(bias_name + '_DequantizeLinear', inputs, 
+                [bias_name + '_dequantized'], axis)
             self.new_nodes.append(dequant_node)
             self.replace_input.append([find_by_name(node.name, self.model.nodes()), 
                 bias_name, bias_name + '_dequantized'])
 
-    def quantize_bias(self, bias_name, input_name, weight_name, new_node_list=[]):
+    def quantize_bias(self, bias_name, input_name, weight_name, beta=1.0):
         '''
         Quantized the bias. Zero Point == 0 and Scale == Input_Scale * Weight_Scale
         '''
@@ -591,7 +608,7 @@ class Quantizer:
 
         # calcuate scale for bias
 
-        bias_scale = input_scale * weight_scale
+        bias_scale = input_scale * weight_scale * beta
 
         # quantize bias
         quantized_data = (np.asarray(bias_data) / bias_scale).round().astype(np.int32)
@@ -683,8 +700,8 @@ class Quantizer:
                 qlinear_node = make_quant_node(weight_name + "_QuantizeLinear",
                         [weight_name, scale_name, zp_name], [weight_name + "_quantized"])
                 dequant_node = make_dquant_node(weight_name + "_DequantizeLinear",
-                            [weight_name + "_QuantizeLinear", scale_name, zp_name], 
-                            [weight_name + "_dequantized"])
+                            [weight_name + "_quantized", scale_name, zp_name], 
+                            [weight_name + "_dequantized"], axis)
                 self.replace_input.append([node, weight_name, dequant_node.output[0]])
                 self.new_nodes.extend([qlinear_node, dequant_node])
             else:
@@ -694,7 +711,7 @@ class Quantizer:
                                                                                axis)
                 inputs = [q_name, scale_name, zp_name]
                 dequant_node = make_dquant_node(weight_name + '_DequantizeLinear',
-                    [q_name, scale_name, zp_name], [weight_name + "_dequantized"])
+                    [q_name, scale_name, zp_name], [weight_name + "_dequantized"], axis)
                 self.new_nodes.append(dequant_node)
 
                 # Replace weight_name with output of DequantizeLinear
@@ -791,7 +808,8 @@ class Quantizer:
             weights = onnx.numpy_helper.to_array(initializer)
         else:
             raise ValueError('Only float type quantization is supported. \
-               Weights {} is {}. '.format(initializer.name, type_to_name[initializer.data_type]))
+                Weights {} is {}.'.format(initializer.name, 
+                    dtype_to_name(dtype_mapping, initializer.data_type)))
         return weights
 
     def _get_quantization_params(self, param_name):
