@@ -15,12 +15,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-from onnxruntime.quantization.quant_utils import QuantizationMode
+from .util import QuantizationMode
 from .operators.base_operator import QuantOperatorBase
 from .operators.qdq_base_operator import QDQOperatorBase
 from .operators.matmul import MatMulInteger, QLinearMatMul, QDQMatMul
 from .operators.attention import AttentionQuant, QDQAttention
-from .operators.embed_layernorm import EmbedLayerNormalizationQuant
+from .operators.embed_layernorm import EmbedLayerNormalizationQuant, QDQEmbedLayerNormalization
 from .operators.gather import GatherConverter, GatherQuant
 from .operators.conv import QLinearConv, ConvInteger, QDQConv
 from .operators.activation import QLinearActivation, QDQRemovableActivation, QDQActivation
@@ -34,6 +34,9 @@ from .operators.pad import QPad, QDQPad
 from .operators.pooling import QLinearPool, QDQPool
 from .operators.direct_q8 import QDQDirect8BitOp, Direct8BitOp, DirectCast
 from .operators.base_operator_cast import CastOperatorBase
+from .operators.argmax import QArgMax
+from .operators.gemm import QLinearGemm, QDQGemm
+from .operators.resize import QResize, QDQResize
 
 CommonOpsRegistry = {"Gather": GatherConverter, \
                      "EmbedLayerNormalization": EmbedLayerNormalizationQuant}
@@ -68,6 +71,9 @@ QLinearOpsRegistry = {
     "Transpose" : Direct8BitOp,
     "Squeeze" : Direct8BitOp,
     "Unsqueeze" : Direct8BitOp,
+    "Resize": QResize,
+    "ArgMax": QArgMax,
+    "Gemm": QLinearGemm,
 }
 QLinearOpsRegistry.update(CommonOpsRegistry)
 
@@ -92,7 +98,10 @@ QDQRegistry = {
     "AveragePool": QDQPool,
     "Unsqueeze" : QDQDirect8BitOp,
     "Concat": QDQConcat,
-    "Split": QDQSplit
+    "Split": QDQSplit,
+    "EmbedLayerNormalization": QDQEmbedLayerNormalization,
+    "Gemm": QDQGemm,
+    "Resize": QDQResize,
 }
 
 CastRegistry = {
