@@ -43,6 +43,7 @@ class TensorFlowAdaptor(Adaptor):
         "AvgPool": "pooling",
         "ConcatV2": "concat",
         "MatMul": "matmul",
+        "BatchMatMul": "matmul",
         "BatchMatMulV2": "matmul",
         "Pad": "pad"
     }
@@ -55,6 +56,7 @@ class TensorFlowAdaptor(Adaptor):
         self.device = self.framework_specific_info['device']
         self.work_dir = os.path.abspath(self.framework_specific_info['workspace_path'])
         self.recipes = deep_get(self.framework_specific_info, 'recipes', {})
+        self.performance_only = deep_get(self.framework_specific_info, 'performance_only', False)
         os.makedirs(self.work_dir, exist_ok=True)
 
         self.pre_optimized_model = None
@@ -66,7 +68,7 @@ class TensorFlowAdaptor(Adaptor):
 
         cfg_yaml_name = "{}.yaml".format(self.__class__.__name__[:-len('Adaptor')].lower())
         self.query_handler = TensorflowQuery(local_config_file=os.path.join(
-            os.path.dirname(__file__), cfg_yaml_name))
+            os.path.dirname(__file__), cfg_yaml_name), performance_only=self.performance_only)
         self.itex_mode = cfg_yaml_name == 'tensorflow_itex.yaml'
         self.qdq_enabled = cfg_yaml_name == 'inteltensorflow.yaml' or \
                                    cfg_yaml_name == 'tensorflow_itex.yaml'
@@ -107,7 +109,7 @@ class TensorFlowAdaptor(Adaptor):
         writer.add_summary(summary, step)
         writer.flush()
 
-    def _pre_hook_for_hvd(self):
+    def _pre_hook_for_hvd(self, dataloader=None):
         import horovod.tensorflow as hvd
         self.hvd = hvd
         self.hvd.init()
@@ -543,13 +545,14 @@ class TensorFlowAdaptor(Adaptor):
                                         bf16_ops=self.bf16_ops,
                                         data_loader=data_loader,
                                         qdq_enabled=self.qdq_enabled,
-                                        new_api=self.new_api).convert()
+                                        new_api=self.new_api,
+                                        performance_only = self.performance_only).convert()
             except Exception: # pragma: no cover
                 from .tf_utils.util import get_model_input_shape
                 batch_size = get_model_input_shape(model)
                 logger.warning(
                         "Fail to forward with batch size={}, set to {} now.".
-                        format(batch_size, batch_size))
+                        format(data_loader.batch_size, batch_size))
                 data_loader.batch(batch_size)
                 self.quantize_config['calib_iteration'] = calib_sampling_size
                 converted_model = GraphConverter(model,
@@ -560,7 +563,8 @@ class TensorFlowAdaptor(Adaptor):
                                         bf16_ops=self.bf16_ops,
                                         data_loader=data_loader,
                                         qdq_enabled=self.qdq_enabled,
-                                        new_api=self.new_api).convert()
+                                        new_api=self.new_api,
+                                        performance_only = self.performance_only).convert()
         else: # pragma: no cover
             if hasattr(data_loader, 'batch_size') and \
               calib_sampling_size % data_loader.batch_size != 0:
@@ -579,7 +583,8 @@ class TensorFlowAdaptor(Adaptor):
                                 bf16_ops=self.bf16_ops,
                                 data_loader=data_loader,
                                 qdq_enabled=self.qdq_enabled,
-                                new_api=self.new_api).convert()
+                                new_api=self.new_api,
+                                performance_only = self.performance_only).convert()
         #just save framework_specific_info feature for recover
         converted_model.q_config.update({'framework_specific_info': \
                                             self.framework_specific_info})
@@ -599,7 +604,8 @@ class TensorFlowAdaptor(Adaptor):
         int8_op_prefix_list = ['QuantizedConv2D', '_QuantizedConv3D', 'QuantizedDepthwise',
                                'QuantizedMaxPool', 'QuantizedAvgPool',
                                'QuantizedConcatV2', 'QuantizedMatMul',
-                               '_QuantizedFusedBatchNorm']
+                               '_QuantizedFusedBatchNorm', '_QuantizedMatMul',
+                               '_QuantizedBatchMatMul']
         from tensorflow.python.framework import dtypes
 
         res = {}
@@ -620,6 +626,8 @@ class TensorFlowAdaptor(Adaptor):
                     origin_op_type = 'FusedBatchNormV3'
                 if origin_op_type == 'Depthwise':
                     origin_op_type = 'DepthwiseConv2dNative'
+                if origin_op_type == 'BatchMatMul':
+                    origin_op_type = 'BatchMatMulV2'
                 res[origin_op_type]['INT8'] += 1
 
             if i.op in fp32_op_list:
@@ -794,6 +802,7 @@ class TensorFlowAdaptor(Adaptor):
                     return True
             return False
 
+
         self.filter_unquantizable_concat(matched_nodes)
 
         copied_matched_nodes = copy.deepcopy(matched_nodes)
@@ -1350,7 +1359,8 @@ class TensorFlowAdaptor(Adaptor):
         converter = GraphConverter(model,
                                    qt_config=quantize_config,
                                    int8_sequences=self.op_wise_sequences,
-                                   fake_quant=True, new_api=self.new_api)
+                                   fake_quant=True, new_api=self.new_api,
+                                   performance_only = self.performance_only)
 
         return converter.convert()
 
@@ -1374,7 +1384,8 @@ class TensorFlowAdaptor(Adaptor):
         from .tf_utils.graph_converter_without_calib import GraphConverterWithoutCalib
         converter = GraphConverterWithoutCalib(model,
                                             recover_config=q_config,
-                                            new_api=self.new_api)
+                                            new_api=self.new_api,
+                                            performance_only=self.performance_only)
 
         return converter.convert_without_calib()
     
@@ -1435,7 +1446,8 @@ class Tensorflow_ITEXAdaptor(TensorFlowAdaptor):
                                     data_loader=data_loader,
                                     itex_mode=self.itex_mode,
                                     qdq_enabled=self.qdq_enabled,
-                                    new_api=self.new_api).convert()
+                                    new_api=self.new_api,
+                                    performance_only = self.performance_only).convert()
             except Exception: # pragma: no cover
                 logger.warning(
                         "Fail to forward with batch size={}, set to {} now.".
@@ -1451,7 +1463,8 @@ class Tensorflow_ITEXAdaptor(TensorFlowAdaptor):
                                 data_loader=data_loader,
                                 itex_mode=self.itex_mode,
                                 qdq_enabled=self.qdq_enabled,
-                                new_api=self.new_api).convert()
+                                new_api=self.new_api,
+                                performance_only = self.performance_only).convert()
         else: # pragma: no cover
             if hasattr(data_loader, 'batch_size') and \
               calib_sampling_size % data_loader.batch_size != 0:
@@ -1471,7 +1484,8 @@ class Tensorflow_ITEXAdaptor(TensorFlowAdaptor):
                                    data_loader=data_loader,
                                    itex_mode=self.itex_mode,
                                    qdq_enabled=self.qdq_enabled,
-                                   new_api=self.new_api).convert()
+                                   new_api=self.new_api,
+                                   performance_only = self.performance_only).convert()
 
         self._dump_model_op_stats(converted_model.graph_def)
 
@@ -1479,13 +1493,14 @@ class Tensorflow_ITEXAdaptor(TensorFlowAdaptor):
 
 class TensorflowQuery(QueryBackendCapability):
 
-    def __init__(self, local_config_file=None):
+    def __init__(self, local_config_file=None, performance_only=False):
         import tensorflow as tf
 
         super().__init__()
         self.version = tf.version.VERSION
         self.cfg = local_config_file
         self.cur_config = None
+        self.performance_only = performance_only
         self._one_shot_query()
 
     def _get_specified_version_cfg(self, data):
@@ -1546,10 +1561,23 @@ class TensorflowQuery(QueryBackendCapability):
         return config
 
     def _one_shot_query(self):
+        # pylint: disable=E1136
         with open(self.cfg) as f:
             content = yaml.safe_load(f)
             try:
                 self.cur_config = self._get_specified_version_cfg(content)
+                if not self.performance_only:
+                    remove_int8_ops = ['FusedBatchNorm', 'FusedBatchNormV2', 'FusedBatchNormV3']
+                    for op in remove_int8_ops:
+                        while op in self.cur_config['ops']['int8']:
+                            self.cur_config['ops']['int8'].remove(op)
+                        if self.cur_config.get('capabilities'):
+                            self.cur_config['capabilities']['int8'].pop(op, None)
+                        pattern = f'Dequantize + {op} + Relu + QuantizeV2'
+                        if self.cur_config.get('patterns'):
+                            while pattern in self.cur_config['patterns']['int8']:
+                                self.cur_config['patterns']['int8'].remove(pattern)
+
             except Exception as e:
                 logger.info("Fail to parse {} due to {}.".format(self.cfg, str(e)))
                 self.cur_config = None
