@@ -56,17 +56,19 @@ from .graph_rewriter.int8.freeze_fake_quant import FreezeFakeQuantOpOptimizer
 from .graph_rewriter.int8.fuse_conv_requantize import FuseConvRequantizeTransformer
 from .graph_rewriter.int8.fuse_matmul_requantize import FuseMatMulRequantizeTransformer
 from .graph_rewriter.int8.fuse_matmul_requantize import FuseMatMulRequantizeDequantizeTransformer
+from .graph_rewriter.int8.fuse_matmul_requantize import FuseMatMulRequantizeNewAPITransformer
+from .graph_rewriter.int8.fuse_matmul_requantize import FuseMatMulRequantizeDequantizeNewAPITransformer
 from .graph_rewriter.int8.scale_propagation import ScaleProPagationTransformer
 from .graph_rewriter.bf16.bf16_convert import BF16Convert
 from .graph_rewriter.int8.post_quantized_op_cse import PostCseOptimizer
+from .graph_rewriter.int8.post_hostconst_converter import PostHostConstConverter
 from .graph_rewriter.int8.meta_op_optimizer import MetaInfoChangingMemOpOptimizer
-from .graph_rewriter.int8.rnn_convert import QuantizedRNNConverter
 from .graph_rewriter.qdq.insert_qdq_pattern import GenerateGraphWithQDQPattern
 from neural_compressor.adaptor.tf_utils.graph_rewriter.generic.insert_print_node import InsertPrintMinMaxNode
 from .graph_util import GraphRewriterHelper as Helper
 
 
-TF_SUPPORTED_MAX_VERSION = '2.9.1'
+TF_SUPPORTED_MAX_VERSION = '2.10.0'
 TF_SUPPORTED_MIN_VERSION = '1.14.0'
 
 logger = logging.getLogger()
@@ -84,7 +86,8 @@ class GraphConverter:
                  fake_quant=False,
                  itex_mode=False,
                  qdq_enabled=False,
-                 new_api=False):
+                 new_api=False,
+                 performance_only=False):
         """Convert graph.
 
         :param model: input tensorflow model.
@@ -145,7 +148,8 @@ class GraphConverter:
         self._itex_model.input_tensor_names = self.input_tensor_names
         self._tmp_graph_def = copy.deepcopy(self.model.graph_def)
         self.new_api = new_api #bool(version1_gte_version2(tf.version.VERSION, '2.8.0'))
-
+        self.performance_only = performance_only
+        self.exclude_node_names = []
     # pylint: disable=no-member
     def _inference(self, model):
         """Run the calibration on the input graph
@@ -155,6 +159,11 @@ class GraphConverter:
         """
         input_tensor = model.input_tensor
         output_tensor = model.output_tensor
+        # TF table initialization: https://github.com/tensorflow/tensorflow/issues/8665
+        node_names = [node.name for node in model.sess.graph.as_graph_def().node]
+        if 'init_all_tables' in node_names:
+            init_table_op = model.sess.graph.get_operation_by_name('init_all_tables')
+            model.sess.run(init_table_op)
 
         logger.info("Start sampling on calibration dataset.")
         for idx, (inputs, labels) in enumerate(self.data_loader):
@@ -185,7 +194,35 @@ class GraphConverter:
                                 feed_dict[tensor] = inputs[name]
                                 break
                 else:
-                    feed_dict = dict(zip(input_tensor, inputs))
+                    # sometimes the input_tensor is not the same order with inputs
+                    # we should check and pair them
+                    def check_shape(tensor, data):
+                        # scalar or 1 dim default True
+                        if tensor.shape == None or \
+                           len(tensor.shape.dims) == 1 or \
+                           not hasattr(data, 'shape'):
+                            return True
+                        tensor_shape = tuple(tensor.shape)
+                        data_shape = tuple(data.shape)
+                        for tensor_dim, data_dim in zip(tensor_shape, data_shape):
+                            if tensor_dim is not None and tensor_dim != data_dim:
+                                return False
+                        return True
+
+                    disorder_tensors = []
+                    disorder_inputs = [] 
+                    for idx, sort_tensor in enumerate(input_tensor):
+                        sort_input = inputs[idx] 
+                        if check_shape(sort_tensor, sort_input):
+                            feed_dict.update({sort_tensor: sort_input}) 
+                        else:
+                            disorder_tensors.append(sort_tensor)
+                            disorder_inputs.append(sort_input)
+                    for i, dis_tensor in enumerate(disorder_tensors):
+                       for j, dis_input in enumerate(disorder_inputs):  
+                           if check_shape(dis_tensor, dis_input):
+                               feed_dict.update({dis_tensor: dis_input})    
+                               break
             _ = model.sess.run(output_tensor, feed_dict) if model.iter_op==[] \
                 else iterator_sess_run(model.sess, model.iter_op, \
                     feed_dict, output_tensor, self.calib_iteration)
@@ -212,7 +249,7 @@ class GraphConverter:
             if version1_gte_version2(tf.version.VERSION, '2.9.0'):
                 is_supported_version = True
                  
-            if tf.version.VERSION == '1.15.0-up3':
+            if version1_eq_version2(tf.version.VERSION, '1.15.0-up3'):
                 is_supported_version = True
                 
         except Exception as e:
@@ -282,13 +319,21 @@ class GraphConverter:
                 model = self.quantize()
 
         if self.itex_mode:
+            self._itex_model.graph_def = \
+                PostHostConstConverter(self._itex_model.graph_def).do_transformation()
+            self._itex_model.graph_def.library.CopyFrom(self.model.graph_def.library)
             return self._itex_model
-        
-        if len(self.bf16_ops) > 0:
+
+        if self.exclude_node_names:
+            self.bf16_ops.extend(self.exclude_node_names)
+       
+        if (len(self.bf16_ops) > 0 and self.performance_only) or \
+           (os.getenv('MIX_PRECISION_TEST') == '1'):
             model = self.bf16_convert()
         post_cse_graph_def = PostCseOptimizer(model.graph_def).do_transformation()
-        post_cse_graph_def.library.CopyFrom(self.model.graph_def.library)
-        model.graph_def = post_cse_graph_def
+        post_hostconst_graph_def = PostHostConstConverter(post_cse_graph_def).do_transformation()
+        post_hostconst_graph_def.library.CopyFrom(self.model.graph_def.library)
+        model.graph_def = post_hostconst_graph_def
 
         if debug:
             model.save(self.output_graph)
@@ -385,11 +430,6 @@ class GraphConverter:
         """
         try:
             self._quantize_graph()
-
-            self._rnn_details = Helper.analysis_rnn_model(self._tmp_graph_def,
-                                                            bf16_ops=self.bf16_ops,
-                                                            fp32_ops=self.fp32_ops)
-            self.quantized_node_info.extend(self._rnn_details.keys())
             self.quantized_node_info = [tuple(i) for i in self.quantized_node_info]
 
             if self.fake_quant:
@@ -414,9 +454,8 @@ class GraphConverter:
                         self.op_wise_config).do_transformation()
 
                 for i in self.quantized_node_info:
-                    frame_name = self._rnn_details[i] if i in self._rnn_details else None
                     sampling_graph_def, output_names = InsertPrintMinMaxNode(
-                        sampling_graph_def, i[0], i[-1], frame_name).do_transformation()
+                        sampling_graph_def, i[0], i[-1]).do_transformation()
                     output_tensor_names.extend(output_names)
                 if self.quantized_node_info:
                     sampling_graph_def.library.CopyFrom(self.model.graph_def.library)
@@ -484,7 +523,7 @@ class GraphConverter:
             self._tmp_model.input_node_names,
             self._tmp_model.output_node_names)
 
-        self._tmp_graph_def, self.quantized_node_info = QuantizeGraphForIntel(
+        self._tmp_graph_def, self.quantized_node_info, exclude_node_names = QuantizeGraphForIntel(
             self._tmp_graph_def,
             self._tmp_model.input_node_names,
             self._tmp_model.output_node_names,
@@ -492,8 +531,9 @@ class GraphConverter:
             self.int8_sequences,
             self.device,
             self.fake_quant,
-            self.new_api).do_transform()
-
+            self.new_api,
+            self.performance_only).do_transform()
+        self.exclude_node_names = exclude_node_names
         self._tmp_graph_def.library.CopyFrom(self.model.graph_def.library)
         if debug:
             self._tmp_model.graph_def = self._tmp_graph_def
@@ -529,11 +569,11 @@ class GraphConverter:
         self._tmp_graph_def, quantizev2_max = FreezeValueTransformer(
             self._tmp_graph_def,
             self._calibration_data,
-            '__max:').do_transformation()
+            '__max:', device=self.device).do_transformation()
         self._tmp_graph_def, quantizev2_min = FreezeValueTransformer(
             self._tmp_graph_def,
             self._calibration_data,
-            '__min:').do_transformation()
+            '__min:', device=self.device).do_transformation()
         self._tmp_graph_def, requant_min_max= FreezeValueTransformer(
             self._tmp_graph_def,
             self._calibration_data,
@@ -546,15 +586,12 @@ class GraphConverter:
         self.scale_info.update(quantizev2_min)
         self.scale_info.update(requant_min_max)
 
-        self._tmp_graph_def = QuantizedRNNConverter(
-            self._tmp_graph_def, self._calibration_data, self._rnn_details).do_transformation()
-
         if 'scale_propagation_max_pooling' in self.recipes and \
                 self.recipes['scale_propagation_max_pooling']:
             self._tmp_graph_def = ScaleProPagationTransformer(
                 self._tmp_graph_def).do_transformation()
 
-        if debug:
+        if debug and not self.new_api:
             self._tmp_graph_def.library.CopyFrom(self.model.graph_def.library)
             self._tmp_model.graph_def = self._tmp_graph_def
             self._tmp_model.save(self._int8_frozen_range_model_path)
@@ -569,20 +606,19 @@ class GraphConverter:
             self.device, self.new_api).do_transformation()
 
         if not self.fake_quant:
-            # TODO Use MatMul and BatchMatMul new API
-            #if self.qdq_enabled:
-            #    self._tmp_graph_def = FuseMatMulRequantizeNewAPITransformer(
-            #        self._tmp_graph_def).do_transformation()
-            #
-            #    self._tmp_graph_def = FuseMatMulRequantizeDequantizeNewAPITransformer(
-            #        self._tmp_graph_def).do_transformation()
-            #else:
-            self._tmp_graph_def = FuseMatMulRequantizeTransformer(
-                self._tmp_graph_def).do_transformation()
-
-            self._tmp_graph_def = FuseMatMulRequantizeDequantizeTransformer(
-                self._tmp_graph_def).do_transformation()
+            if self.qdq_enabled:
+                self._tmp_graph_def = FuseMatMulRequantizeNewAPITransformer(
+                    self._tmp_graph_def).do_transformation()
+            
+                self._tmp_graph_def = FuseMatMulRequantizeDequantizeNewAPITransformer(
+                    self._tmp_graph_def).do_transformation()
+            else:
+                self._tmp_graph_def = FuseMatMulRequantizeTransformer(
+                            self._tmp_graph_def).do_transformation()
 
+                self._tmp_graph_def = FuseMatMulRequantizeDequantizeTransformer(
+                            self._tmp_graph_def).do_transformation()
+        
         self._tmp_graph_def = StripUnusedNodesOptimizer(
             self._tmp_graph_def,
             self._tmp_model.input_node_names,
@@ -596,9 +632,10 @@ class GraphConverter:
         self._tmp_graph_def = FoldBatchNormNodesOptimizer(
             self._tmp_graph_def).do_transformation()
 
-        if 'scale_propagation_concat' in self.recipes and self.recipes['scale_propagation_concat']:
+        if self.performance_only or ('scale_propagation_concat' in self.recipes \
+             and self.recipes['scale_propagation_concat']):
             self._tmp_graph_def = RerangeQuantizedConcat(self._tmp_graph_def,
-                                                     self.device).do_transformation()
+                self.device, performance_only=self.performance_only).do_transformation()
 
         self._tmp_graph_def = MetaInfoChangingMemOpOptimizer(
             self._tmp_graph_def).do_transformation()
@@ -676,13 +713,8 @@ class GraphConverter:
                                                self.int8_sequences,
                                                self.device,
                                                self.fake_quant,
-                                               self.new_api).get_quantized_nodes()
-
-        self._rnn_details = Helper.analysis_rnn_model(self._tmp_graph_def,
-                                                      bf16_ops=self.bf16_ops,
-                                                      fp32_ops=self.fp32_ops)
-        self.quantized_node_info.extend(self._rnn_details.keys())
-        self.quantized_node_info = [tuple(i) for i in self.quantized_node_info]
+                                               self.new_api,
+                                               self.performance_only).get_quantized_nodes()
 
         if self.itex_mode:
             self.quantized_node_info.extend(self._search_y_pattern_for_itex())
@@ -706,9 +738,8 @@ class GraphConverter:
                                     self.op_wise_config).do_transformation()
 
         for i in self.quantized_node_info:
-            frame_name = self._rnn_details[i] if i in self._rnn_details else None
             sampling_graph_def, output_names = InsertPrintMinMaxNode(
-                sampling_graph_def, i[0], i[-1], frame_name).do_transformation()
+                sampling_graph_def, i[0], i[-1]).do_transformation()
             output_tensor_names.extend(output_names)
 
         if self.quantized_node_info:
@@ -727,7 +758,7 @@ class GraphConverter:
         self._tmp_graph_def = GenerateGraphWithQDQPattern(
               self._tmp_model, self._calibration_data,
               self.op_wise_config, self.fake_quant, self.fp32_ops, self.bf16_ops, \
-              self.quantized_node_info, self.device).do_transformation()
+              self.quantized_node_info, self.device, self.performance_only).do_transformation()
 
     def _convert_qdq(self):
         if self.itex_mode:
@@ -760,15 +791,16 @@ class GraphConverter:
             self._itex_model.graph_def = self._tmp_graph_def
             self._itex_model.graph_def.library.CopyFrom(self.model.graph_def.library)
         else:
-            self._tmp_graph_def = OptimizeQDQGraph(self._tmp_graph_def,
+            self._tmp_graph_def, exclude_node_names = OptimizeQDQGraph(self._tmp_graph_def,
                                                    self._tmp_model.input_node_names,
                                                    self._tmp_model.output_node_names,
                                                    self.op_wise_config,
                                                    self.int8_sequences,
                                                    self.device,
                                                    self.fake_quant,
-                                                   self.new_api).do_transform()
-
+                                                   self.new_api,
+                                                   self.performance_only).do_transform()
+            self.exclude_node_names=exclude_node_names
             if len(self._calibration_data) > 0:
                 self._freeze_requantization_ranges(self._kl_op_dict)
                 self._fuse_requantize_with_fused_quantized_node()
