@@ -43,14 +43,18 @@ from .convert_nan_to_random import ConvertNanToRandom
 from .expanddims_optimizer import ExpandDimsOptimizer
 from .fetch_weight_from_reshape import FetchWeightFromReshapeOptimizer
 from .fuse_decomposed_bn import FuseDecomposedBNOptimizer
-
+from .dilated_contraction import DilatedContraction
 
 class PreOptimization():
     def __init__(self, model, optimization, new_api):
         self.model = model
         self.optimization = optimization
+        # Table initialization should disable grappler dependency and pruning pass
+        node_names = [node.name for node in model.graph_def.node]
+        if 'init_all_tables' in node_names:
+            self.optimization['dependency'] = False
+            self.optimization['pruning'] = False
         self.new_api = new_api
-
         self.analyzer = GraphAnalyzer()
         self.analyzer.graph = model.graph_def
         self.analyzer.parse_graph()
@@ -108,10 +112,19 @@ class PreOptimization():
 
         self._tmp_graph_def = SplitSharedInputOptimizer(self._tmp_graph_def).do_transformation()
 
+        # Put FuseDecomposedBNOptimizer before GraphFoldConstantOptimizer
+        # The 'Sub' op in the small decomposed ops of BN will be converted to const by GraphFoldConstantOptimizer.
+        # Then the FuseDecomposedBNOptimizer can't fuse the small decomposed ops to BN.
+        if self.new_api:
+            self._tmp_graph_def = FuseDecomposedBNOptimizer(self._tmp_graph_def).do_transformation()
+
         # disable fold constant for itex qdq mode
         if not itex_mode:
             self._tmp_graph_def = GraphFoldConstantOptimizer(self._tmp_graph_def).do_transformation()
 
+        if not self.new_api:
+            self._tmp_graph_def = FuseDecomposedBNOptimizer(self._tmp_graph_def).do_transformation()
+
         self._tmp_graph_def = FuseColumnWiseMulOptimizer(self._tmp_graph_def).do_transformation()
 
         self._tmp_graph_def = StripUnusedNodesOptimizer(self._tmp_graph_def,
@@ -121,18 +134,12 @@ class PreOptimization():
 
         self._tmp_graph_def = GraphCseOptimizer(self._tmp_graph_def).do_transformation()
 
-        self._tmp_graph_def = FuseDecomposedBNOptimizer(self._tmp_graph_def).do_transformation()
-
         self._tmp_graph_def = FoldBatchNormNodesOptimizer(
             self._tmp_graph_def).do_transformation()
 
         self._tmp_graph_def = RenameBatchNormOptimizer(
             self._tmp_graph_def).do_transformation()
 
-        #TODO we should handle all control ops elegantly not bypass it.
-        self._tmp_graph_def, excluded_node_names = UpdateEnterOptimizer(
-            self._tmp_graph_def).do_transformation()
-
         self._tmp_graph_def = ConvertLeakyReluOptimizer(
             self._tmp_graph_def).do_transformation()
 
@@ -164,7 +171,9 @@ class PreOptimization():
         self._tmp_graph_def = ConvertNanToRandom(
             self._tmp_graph_def).do_transformation()
 
-        self._excluded_node_names.extend(excluded_node_names)
+        if self.new_api:
+            self._tmp_graph_def = DilatedContraction(
+                self._tmp_graph_def).do_transformation()
         self._tmp_graph_def.library.CopyFrom(self.model.graph_def.library)
 
         origin_model.graph_def = self._tmp_graph_def
