@@ -36,7 +36,7 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
     """
 
     def __init__(self, model, calibration_data, op_wise_config, fake_quant, fp32_ops,
-                 bf16_ops, quantized_nodes, device):
+                 bf16_ops, quantized_nodes, device, performance_only):
         super().__init__(model)
         self.data = calibration_data
         self.op_wise_config = op_wise_config
@@ -45,6 +45,7 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
         self.bf16_ops = bf16_ops
         self.quantized_nodes = quantized_nodes
         self.device = device
+        self.performance_only = performance_only
 
         self.node_name_mapping = {}
         for node in self.model.graph_def.node:
@@ -95,7 +96,7 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
         self.g_weight.graph = self.g.dump_graph()
         self.graph_info = self.g_weight.parse_graph()
         target_nodes = self.g_weight.query_fusion_pattern_nodes(
-               [["Conv2D", "Conv3D", "DepthwiseConv2dNative", "MatMul", "BatchMatMulV2"]])
+               [["Conv2D", "Conv3D", "DepthwiseConv2dNative", "MatMul", "BatchMatMul", "BatchMatMulV2"]])
         for i in target_nodes:
             if i[0] not in quantizable_op_names:
                 continue
@@ -105,7 +106,13 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
 
             computational_node = self.graph_info[computational_node_name].node
             weight_name = computational_node.input[1]
-            weight_node = self.graph_info[weight_name].node
+            if re.search(r"\w+:\d+", weight_name):
+                weight_node = self.graph_info[weight_name.rsplit(':', 1)[0]].node
+            else:
+                weight_node = self.graph_info[weight_name].node
+            if weight_node.op == 'Enter':
+                continue
+
             if computational_node_name in self.op_wise_config.keys():
                 op_wise_cfg = self.op_wise_config[computational_node_name]
                 per_channel = op_wise_cfg[0]
@@ -126,13 +133,20 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
     def _check_op_list(self, node_type):
         op_list = ("ConcatV2", "Conv2D", "Conv3D", "DepthwiseConv2D", "QuantizeV2", "DepthwiseConv2dNative",
                    "MaxPool", "MaxPool3D", "FusedBatchNormV3", "Requantize", "RequantizePerChannel", "AvgPool", "Pad",
-                   "CropAndResize", "Dequantize", "Mean", "MatMul", "BatchMatMulV2", "FakeQuantWithMinMaxVars")
+                   "CropAndResize", "Dequantize", "Mean", "MatMul", "BatchMatMul",
+                   "BatchMatMulV2", "FakeQuantWithMinMaxVars")
         return any([node_type.find(i) != -1 for i in op_list])
 
     def _find_relu_node(self, node):
-        if node.op in ("Relu", "Relu6", "Elu") or \
+        if (node.op in ("Relu", "Relu6", "Elu") or \
             (node.op.find("AndRelu") != -1 and \
-            ('alpha' not in node.attr or ('alpha' in node.attr and node.attr['alpha'].f == 0))):
+            ('alpha' not in node.attr or ('alpha' in node.attr and node.attr['alpha'].f == 0)))) \
+                and (node.op != "Relu"
+                     or not self.performance_only
+                     or self.node_name_mapping \
+                        [Helper.node_name_from_input(node.input[0])].op.find("FusedBatchNorm") == -1
+                     or self.node_name_mapping \
+                        [Helper.node_name_from_input(node.input[0])].attr['is_training'].b):
             return True
         elif 'T' in node.attr and node.attr['T'].type in (dtypes.quint8, dtypes.uint8):
             return True
@@ -143,11 +157,21 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
               ('alpha' in node.attr and node.attr['alpha'].f > 0)):
             return False
         elif self._check_op_list(node.op):
-            if "split:" in node.input[0]:
-                input_node = self.node_name_mapping[node.input[0].rsplit(':', 1)[0]]
+            if node.op == 'ConcatV2':
+                find_relu = False
+                for i in range(0,node.attr['N'].i):
+                    if re.search(r"\w+:\d+", node.input[i]):
+                        input_node = self.node_name_mapping[node.input[i].rsplit(':', 1)[0]]
+                    else:
+                        input_node = self.node_name_mapping[node.input[i]]
+                    find_relu |= self._find_relu_node(input_node)
+                return find_relu
             else:
-                input_node = self.node_name_mapping[node.input[0]]
-            return self._find_relu_node(input_node)
+                if re.search(r"\w+:\d+", node.input[0]):
+                    input_node = self.node_name_mapping[node.input[0].rsplit(':', 1)[0]]
+                else:
+                    input_node = self.node_name_mapping[node.input[0]]
+                return self._find_relu_node(input_node)
         else:
             return False
 
@@ -157,9 +181,10 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
             if each_input_name[0] == '^':
                 continue
 
-            if self.node_name_mapping[original_node.name].op == "MatMul" or \
-               self.node_name_mapping[original_node.name].op == "BatchMatMulV2":
+            if self.node_name_mapping[original_node.name].op == "MatMul":
                 dtype = dtypes.quint8
+            elif self.node_name_mapping[original_node.name].op == "BatchMatMulV2":
+                dtype = dtypes.qint8
             else:
                 input_node_name = Helper.node_name_from_input(each_input_name)
                 if input_node_name in self.graph_info:
@@ -178,7 +203,8 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
                                                     namespace_prefix,
                                                     each_input_name,
                                                     is_asymmetric,
-                                                    dtype)
+                                                    dtype,
+                                                    device=self.device)
 
 
     def _insert_qdq_pattern_for_concatv2(self, original_node, is_asymmetric):
@@ -192,27 +218,29 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
                                                     original_input_name,
                                                     is_asymmetric,
                                                     dtypes.quint8,
-                                                    input_idx)
+                                                    input_idx,
+                                                    device=self.device)
             input_idx += 1
 
 
     def _insert_qdq_pattern_for_each_input(self, op_name, namespace_prefix,
                                            input_name, is_asymmetric,
-                                           dtype=dtypes.quint8, input_index=0):
+                                           dtype=dtypes.quint8, input_index=0,
+                                           device='cpu'):
         """Takes one float input to an op, and converts it to quantized form."""
         unique_input_name = input_name.replace(":", "__port__").replace("^", "__hat__")
         min_input_name = namespace_prefix + "_min_" + unique_input_name
         max_input_name = namespace_prefix + "_max_" + unique_input_name
         quantize_input_name = namespace_prefix + "_quantize_" + unique_input_name
 
-        reshape_dims_name = namespace_prefix + "_reshape_dims"
-        reduction_dims_name = namespace_prefix + "_reduction_dims"
+        reshape_dims_name = namespace_prefix + "_reshape_dims" + unique_input_name
+        reduction_dims_name = namespace_prefix + "_reduction_dims" + unique_input_name
 
         if self.fake_quant: # pragma: no cover
             min_node = Helper.create_constant_node(
-                min_input_name, -1., dtypes.float32)
+                min_input_name, -1., dtypes.float32, device="cpu")
             max_node = Helper.create_constant_node(
-                max_input_name, 1., dtypes.float32)
+                max_input_name, 1., dtypes.float32, device="cpu")
             quant_v2_node = Helper.create_node(
                 "QuantizeV2", quantize_input_name,
                 [input_name, min_input_name, max_input_name])
@@ -266,6 +294,10 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
             Helper.set_attr_dtype(max_input_node, "T", dtypes.float32)
             Helper.set_attr_dtype(max_input_node, "Tidx", dtypes.int32)
             Helper.set_attr_bool(max_input_node, "keep_dims", False)
+            
+            if "BatchMatMul" in self.graph_info[op_name].node.op:
+                min_input_node.input.append("^" + input_name)
+                max_input_node.input.append("^" + input_name)
 
             quant_v2_node = Helper.create_node("QuantizeV2", quantize_input_name,
                 [input_name, min_input_name, max_input_name])
@@ -318,7 +350,7 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
        
         # The weight node of BatchMatMul may have no value
         if 'value' in weight_node.attr and \
-           host_op_type in ("Conv2D", "MatMul", "BatchMatMulV2", "Conv3D"):
+           host_op_type in ("Conv2D", "MatMul", "BatchMatMul", "BatchMatMulV2", "Conv3D"):
             float_tensor = tensor_util.MakeNdarray(weight_node.attr["value"].tensor)
             if per_channel:
                 if host_op_type == 'Conv3D':
@@ -379,9 +411,13 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
             max_value = np.max(min_max_values[computational_node.name+'__max'])
 
         min_node = Helper.create_constant_node(min_name, min_value,
-                                                            dtypes.float32, device=device)
+                                                            dtypes.float32, device="cpu")
         max_node = Helper.create_constant_node(max_name, max_value,
-                                                            dtypes.float32, device=device)
+                                                            dtypes.float32, device="cpu")
+        if "BatchMatMul" in host_op_type and "BatchMatMul" not in weight_node.op:
+            min_node.input.append("^" + weight_node.name)
+            max_node.input.append("^" + weight_node.name)
+
         quant_node = Helper.create_node(
                 "QuantizeV2", qint8_const_name + '_quant',
                 [weight_node.name, min_name, max_name])
@@ -406,7 +442,10 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
             else:
                 Helper.set_attr_int(quant_node, 'axis', -1)
                 Helper.set_attr_int(dequant_node, 'axis', -1)
-
+        if host_op_type == 'DepthwiseConv2dNative':
+            Helper.set_attr_int(quant_node, 'axis', 2)
+            Helper.set_attr_int(dequant_node, 'axis', 2)
+        
         self.g_weight.add_node(quant_node, weight_node.name, [])
         self.g_weight.add_node(min_node, None, [quant_node.name])
         self.g_weight.add_node(max_node, None, [quant_node.name])
@@ -422,12 +461,12 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
             return True
 
         #TODO Remove below two lines once the TF enabled the QuantizedMatMul while
-        # transpose_a/transpose_a could be set to True.
+        # transpose_a could be set to True.
         if self.graph_info[matched_node_name].node.op == "MatMul":
-            if self.graph_info[matched_node_name].node.attr["transpose_a"].b == True or \
-               self.graph_info[matched_node_name].node.attr["transpose_b"].b == True:
+            if self.graph_info[matched_node_name].node.attr["transpose_a"].b == True:
                 return True
-
+        if "FusedBatchNorm" in self.graph_info[matched_node_name].node.op:
+            return True
         return False
 
 
