@@ -22,8 +22,10 @@ import tensorflow as tf
 
 from tensorflow.core.framework import graph_pb2
 from tensorflow.python.framework import dtypes
-from .quantize_graph_common import QuantizeGraphHelper as helper
-
+from neural_compressor.adaptor.tf_utils.quantize_graph_common import QuantizeGraphHelper as helper
+from neural_compressor.adaptor.tf_utils.util import version1_gt_version2
+from neural_compressor.adaptor.tf_utils.util import version1_lt_version2
+from neural_compressor.adaptor.tf_utils.util import version1_eq_version2
 
 class QuantizeGraphBase():
     """
@@ -77,11 +79,16 @@ class QuantizeNodeBase():
         self.patterns =  kwargs['patterns']
         self.remove_redundant_quant_flag = kwargs['remove_redundant_quant_flag']
         self.fake_quant = kwargs['fake_quant'] if 'fake_quant' in kwargs else False
+        self.frame_info = kwargs['frame_info'] if 'frame_info' in kwargs else None
         self.per_channel, self.is_asymmetric = kwargs['op_wise_cfg'][0], kwargs['op_wise_cfg'][2]
+        self.op_wise_config_name_list = kwargs['op_wise_config_name_list']
         self.weight_bit = kwargs['op_wise_cfg'][3]
         self.start_node_name = kwargs['start_node_name']
         self.device = kwargs['device']
-        self.enable_s8 = bool(tf.version.VERSION >= '2.1.0' or \
+        self.new_api = kwargs['new_api']
+        self.performance_only = kwargs['performance_only']
+        self.enable_s8 = bool(version1_gt_version2(tf.version.VERSION, '2.1.0') or \
+                    version1_eq_version2(tf.version.VERSION, '2.1.0') or \
             tf.version.VERSION.find('1.15.0-up') != -1)
 
     def apply_the_transform(self):
@@ -96,6 +103,55 @@ class QuantizeNodeBase():
         """
         pass
 
+    def _insert_dummy_biasadd(self, match_node_name, matched_node): # pragma: no cover
+         target_node_name = matched_node.node.name
+         matmul_a_node_name = helper.node_name_from_input(matched_node.node.input[0])
+         matmul_a_node = self.node_name_mapping[matmul_a_node_name].node
+         matmul_b_node_name = helper.node_name_from_input(matched_node.node.input[1])
+         matmul_b_node = self.node_name_mapping[matmul_b_node_name].node
+
+         if matmul_a_node.op == 'Const' and matmul_b_node.op != 'Const':
+             pass
+         else:
+             from neural_compressor.adaptor.tf_utils.graph_util import GraphAnalyzer
+             g = GraphAnalyzer()
+             g.graph = self.input_graph
+             graph_info = g.parse_graph()
+             next_node_names = graph_info[matched_node.node.name].outputs
+             bias_node_name = target_node_name + '_dummy_biasadd'
+             bias_const_node_name = target_node_name + '_fake_const'
+
+             if matched_node.node.op == 'MatMul':
+                 t_b_index = 0 if matched_node.node.attr['transpose_b'].b else 1
+             if matched_node.node.op in ('Conv2D' or 'DepthwiseConv2dNative') and \
+                matched_node.node.attr['data_format'].s == b'NHWC':
+                 t_b_index = 3
+             elif matched_node.node.op in ('Conv2D' or 'DepthwiseConv2dNative')  and \
+                matched_node.node.op.attr['data_format'].s == b'NCHW':
+                 t_b_index = 1
+             elif matched_node.node.op == 'Conv3D' and matched_node.node.attr['data_format'].s == b'NDHWC':
+                 t_b_index = 4
+             elif matched_node.node.op == 'Conv3D' and matched_node.node.attr['data_format'].s == b'NCDHW':
+                 t_b_index = 1
+             bias_add_length = matmul_b_node.attr['value'].tensor.tensor_shape.dim[t_b_index].size
+
+             bias_add_content = [0.] * bias_add_length
+
+             bias_const_node = helper.create_constant_node(
+                 bias_const_node_name, bias_add_content, dtypes.float32, shape=[bias_add_length])
+             bias_node = helper.create_node('BiasAdd', bias_node_name, [target_node_name, bias_const_node_name])
+             helper.set_attr_dtype(bias_node, "T", dtypes.float32)
+
+             g.add_node(bias_node, target_node_name, next_node_names)
+             g.add_node(bias_const_node, None, [bias_node_name])
+             self.input_graph = g.dump_graph()
+             self._parse_graph(self.input_graph)
+             new_match_node_name=match_node_name[:1]+[bias_node_name]+match_node_name[1:]
+             new_match_node_name=match_node_name[:1]+[bias_node_name]+match_node_name[1:]
+
+             return new_match_node_name
+
+
     def _is_match(self, patterns):
         """Detect the rule matched nodes collections.
 
@@ -116,12 +172,13 @@ class QuantizeNodeBase():
                      and not self.enable_s8)
                     ) and not self._find_relu_node(cur_node):
                     continue
-
+               
                 for sub_rule in patterns:
                     if v != sub_rule[0]:
                         continue
 
                     sub_rule_len = len(sub_rule)
+                    check_hardswish = True if sub_rule_len > 4  else False
                     self.logger.debug("Try to apply rule: {}".format(sub_rule))
 
                     cur_node_name = list(self.node_name_mapping.keys())[k]
@@ -129,7 +186,7 @@ class QuantizeNodeBase():
                     matched_node_name.clear()
 
                     matched_node_name.append(cur_node_name)
-
+                    count = 0
                     while sub_rule_len > 1:
                         if not self.node_name_mapping[cur_node_name].output:
                             self.logger.debug("Fail to match {}".format(sub_rule))
@@ -138,12 +195,26 @@ class QuantizeNodeBase():
                         next_node_name = self.node_name_mapping[
                             cur_node_name].output[0]
 
+                        is_shared_output = True if len(
+                            self.node_name_mapping[cur_node_name].output
+                        ) > 1 else False
+
+                        add_op_quantizable = True 
+                        is_hardswish = False
+                        if is_shared_output:  # pragma: no cover
+                            if next_node_name.find('hard_swish') != -1:
+                                self.logger.debug("Find Hard Swish pattern ......")
+                                is_hardswish = True
+                                count = count + 1
+                                if next_node_name.find('add') == -1:
+                                    next_node_name = self.node_name_mapping[
+                                    cur_node_name].output[1]
+                            else:
+                                add_op_quantizable = False
                         next_node_op = self.node_name_mapping[
                             next_node_name].node.op
 
-                        add_op_quantizable = True
-
-                        if next_node_op in ("Add", "AddN"):
+                        if next_node_op in ("Add", "AddV2", "AddN"):
                             next_node = self.node_name_mapping[
                                 next_node_name].node
                             next_node_inputs = list(next_node.input)
@@ -159,37 +230,56 @@ class QuantizeNodeBase():
                                     add_op_quantizable = False
                                     break
 
-                        is_shared_output = True if len(
-                            self.node_name_mapping[cur_node_name].output
-                        ) > 1 else False
-
-                        if add_op_quantizable and not is_shared_output and \
-                                next_node_op == sub_rule[1 - sub_rule_len]:
-                            matched_node_name.append(next_node_name)
-                            sub_rule_len -= 1
-                            cur_node_name = next_node_name
+                        if add_op_quantizable and next_node_op == sub_rule[1 - sub_rule_len]:
+                            if not is_shared_output: 
+                                matched_node_name.append(next_node_name)
+                                sub_rule_len -= 1
+                                cur_node_name = next_node_name
+                            elif is_hardswish and self.new_api: # pragma: no cover
+                                matched_node_name.append(next_node_name)
+                                sub_rule_len -= 1
+                                cur_node_name = next_node_name
+                            else:
+                                matched_node_name.clear()
+                                self.logger.debug("Fail to match {}.".format(sub_rule))
+                                break
                         else:
                             matched_node_name.clear()
                             self.logger.debug("Fail to match {}.".format(sub_rule))
                             break
 
                     if sub_rule_len == 1:
-                        self.logger.debug("Match {} on nodes {}.".
+                         if check_hardswish and sub_rule[-1] == 'Mul' and \
+                            sub_rule[-2] == 'Mul' and sub_rule[-3] == 'Relu6' and \
+                            sub_rule[-4] == 'Add' and count != 1:
+                              matched_node_name.clear()
+                              self.logger.debug("Fail to match {}.".format(sub_rule))
+                              break
+                         self.logger.debug("Match {} on nodes {}.".
                                           format(sub_rule, matched_node_name))
-                        return sub_rule, matched_node_name
+                         return sub_rule, matched_node_name
 
         return None, None
 
     def _need_to_check(self, node_type):
-        op_list = ("ConcatV2", "Conv2D", "DepthwiseConv2D", "QuantizeV2", "DepthwiseConv2dNative",
-                   "MaxPool", "Requantize", "RequantizePerChannel", "AvgPool", "Pad",
-                   "CropAndResize", "Dequantize", "Mean", "MatMul", "FakeQuantWithMinMaxVars")
+        op_list = ("ConcatV2", "Conv2D", "Conv3D", "DepthwiseConv2D", "QuantizeV2", "DepthwiseConv2dNative",
+                   "MaxPool", "MaxPool3D", "FusedBatchNormV3", "Requantize", "RequantizePerChannel", "AvgPool", "Pad",
+                   "CropAndResize", "Dequantize", "Mean", "MatMul", "BatchMatMulV2", "FakeQuantWithMinMaxVars")
         return any([node_type.find(i) != -1 for i in op_list])
 
     def _find_relu_node(self, node):
-        if node.op in ("Relu", "Relu6") or \
+        #if node.op.find("HardSwish") != -1:
+        #    return False
+        if (node.op in ("Relu", "Relu6") or \
             (node.op.find("AndRelu") != -1 and \
-            ('alpha' not in node.attr or ('alpha' in node.attr and node.attr['alpha'].f == 0))):
+            ('alpha' not in node.attr or ('alpha' in node.attr and node.attr['alpha'].f == 0)))) \
+                and (node.op != "Relu"
+                     or not self.new_api
+                     or not self.performance_only
+                     or self.node_name_mapping \
+                        [helper.node_name_from_input(node.input[0])].node.op.find("FusedBatchNorm") == -1
+                     or self.node_name_mapping \
+                        [helper.node_name_from_input(node.input[0])].node.attr['is_training'].b):
             return True
         elif 'T' in node.attr and node.attr['T'].type in (dtypes.quint8, dtypes.uint8):
             return True
@@ -250,6 +340,8 @@ class QuantizeNodeBase():
                                              add_op_function):
         quantized_op_name = original_node.name + "_eightbit_quantized"
         quantized_op_type = "Quantized" + original_node.op
+        if version1_gt_version2(tf.version.VERSION, '2.7.0') and original_node.op == "MaxPool3D":
+            quantized_op_type = "_Quantized" + original_node.op
         all_input_names = self._add_eightbit_prologue_nodes(original_node.name)
         quantized_op_node = helper.create_node(quantized_op_type,
                                                quantized_op_name,
@@ -290,6 +382,8 @@ class QuantizeNodeBase():
                     dtype = dtypes.quint8 if self._find_relu_node(
                         self.node_name_mapping[original_node].node
                     ) else dtypes.qint8
+                    if 'FusedBatchNorm' in self.node_name_mapping[original_node].node.op:
+                        dtype = dtypes.qint8
             quantize_input_name, min_input_name, max_input_name = (
                 self._eightbitize_input_to_node(namespace_prefix,
                                                 each_input_name,
@@ -310,9 +404,60 @@ class QuantizeNodeBase():
                 all_input_names.append(original_input_name)
         return all_input_names
 
+    def _add_eightbit_prologue_nodes_for_enter(self, original_node, enter_node=None):
+        namespace_prefix = original_node + "_eightbit"
+        reshape_dims_name, reduction_dims_name = self._add_common_quantization_nodes(
+            namespace_prefix, helper.node_name_from_input(
+                self.node_name_mapping[original_node].node.input[0]), enter_node=enter_node)
+        input_names = []
+        min_max_names = []
+        enter_input_name = self.node_name_mapping[original_node].node.input[1]
+        if enter_input_name[0] != '^':
+            if self.node_name_mapping[original_node].node.op == "MatMul":
+                # mkl ops _MklQuantizedMatMulWithBiasAndRelu|AndRequantize
+                # requires the T1 data type as quint8
+                dtype = dtypes.quint8
+            else:
+                input_node_name = helper.node_name_from_input(enter_input_name)
+                if input_node_name in self.output_node_maps:
+                    if self.output_node_maps[input_node_name].op == "Dequantize":
+                        dtype = dtypes.DType(
+                            self.output_node_maps[input_node_name].attr["T"].type)
+                    elif self._find_relu_node(self.node_name_mapping[original_node].node):
+                        dtype = dtypes.quint8
+                    else:
+                        dtype = dtypes.qint8
+                else:
+                    dtype = dtypes.quint8 if self._find_relu_node(
+                        self.node_name_mapping[original_node].node
+                    ) else dtypes.qint8
+                    if 'FusedBatchNorm' in self.node_name_mapping[original_node].node.op:
+                        dtype = dtypes.qint8
+            quantize_input_name, min_input_name, max_input_name = (
+                self._eightbitize_input_to_node(namespace_prefix,
+                                                enter_input_name,
+                                                reshape_dims_name,
+                                                reduction_dims_name,
+                                                dtype=dtype))
+            input_names.append(quantize_input_name)
+            min_max_names.append(min_input_name)
+            min_max_names.append(max_input_name)
+
+        all_input_names = []
+        all_input_names.extend(input_names)
+        if min_max_names:
+            all_input_names.extend(min_max_names)
+
+        for original_input_name in self.node_name_mapping[
+                original_node].node.input:
+            if original_input_name[0] == '^':
+                all_input_names.append(original_input_name)
+        return all_input_names
+
     def _add_common_quantization_nodes(self,
                                        namespace_prefix,
-                                       control_input_names=None):
+                                       control_input_names=None,
+                                       enter_node=None):
         """Builds constant nodes needed for quantization of inputs."""
         reshape_dims_name = namespace_prefix + "_reshape_dims"
         reduction_dims_name = namespace_prefix + "_reduction_dims"
@@ -320,12 +465,35 @@ class QuantizeNodeBase():
         reshape_dims_node = helper.create_constant_node(
             reshape_dims_name, -1, dtypes.int32, [1])
 
+        if enter_node: # pragma: no cover
+            reshape_dims_enter_node = helper.create_node(
+                'Enter', reshape_dims_name+'_enter', [reshape_dims_name])
+            helper.set_attr_string(reshape_dims_enter_node,
+                                   'frame_name', enter_node.attr['frame_name'].s)
+            helper.set_attr_dtype(reshape_dims_enter_node, 'T', dtypes.int32)
+            helper.set_attr_bool(reshape_dims_enter_node, 'is_constant', True)
+            helper.set_attr_int(reshape_dims_enter_node, 'parallel_iterations', \
+                                enter_node.attr['parallel_iterations'].i)
+            self.add_output_graph_node(reshape_dims_enter_node)
+
         self.add_output_graph_node(reshape_dims_node)
         reduction_dims_node = helper.create_constant_node(
             reduction_dims_name, 0, dtypes.int32, [1])
 
+        if enter_node: # pragma: no cover
+            reduction_dims_enter_node = helper.create_node(
+                'Enter', reduction_dims_name+'_enter', [reduction_dims_name])
+            helper.set_attr_string(reduction_dims_enter_node,
+                                   'frame_name', enter_node.attr['frame_name'].s)
+            helper.set_attr_dtype(reduction_dims_enter_node, 'T', dtypes.int32)
+            helper.set_attr_bool(reduction_dims_enter_node, 'is_constant', True)
+            helper.set_attr_int(reduction_dims_enter_node, 'parallel_iterations', \
+                                enter_node.attr['parallel_iterations'].i)
+            self.add_output_graph_node(reduction_dims_enter_node)
+
         self.add_output_graph_node(reduction_dims_node)
-        return reshape_dims_name, reduction_dims_name
+        return reshape_dims_enter_node.name if enter_node else reshape_dims_name, reduction_dims_enter_node.name \
+               if enter_node else reduction_dims_name
 
     def add_output_graph_node(self, output_node):
         """Inserts one node into the new graph."""
@@ -459,7 +627,7 @@ class QuantizeNodeBase():
             # Add a RequantizationRange node for finding the min and max values.
             requant_range_node = helper.create_node(
                 "RequantizationRangePerChannel"
-                if self.per_channel else "RequantizationRange",
+                if self.per_channel or original_node.op == 'DepthwiseConv2dNative' else "RequantizationRange",
                 original_node.name + "_eightbit_requant_range", quantized_outputs)
 
             if self.per_channel:
@@ -493,7 +661,8 @@ class QuantizeNodeBase():
             ]
 
         requantize_node = helper.create_node(
-            "RequantizePerChannel" if self.per_channel else "Requantize",
+            "RequantizePerChannel" if self.per_channel or original_node.op == 'DepthwiseConv2dNative' \
+            else "Requantize",
             original_node.name + "_eightbit_requantize",
             quantized_outputs + min_max_inputs)
         if self.per_channel:
@@ -582,11 +751,23 @@ class QuantizeNodeBase():
     def _intel_cpu_quantize_weight_eightbit(self,
                                             parent,
                                             input_node,
-                                            per_channel):
-        qint8_const_node, min_node, max_node = helper.generate_quantized_weight_node(
-            parent, input_node, per_channel, self.weight_bit, self.device)
-        self.add_output_graph_node(qint8_const_node)
-        self.add_output_graph_node(min_node)
-        self.add_output_graph_node(max_node)
+                                            per_channel,
+                                            enter_node=None):
+        qint8_const_node, min_node, max_node, qint8_const_enter_node, min_enter_node, max_enter_node = \
+            helper.generate_quantized_weight_node(parent, input_node, per_channel,
+                                                  self.weight_bit, self.device, enter_node)
+        if enter_node: # pragma: no cover
+            self.add_output_graph_node(qint8_const_node)
+            self.add_output_graph_node(qint8_const_enter_node)
+            self.add_output_graph_node(min_node)
+            self.add_output_graph_node(min_enter_node)
+            self.add_output_graph_node(max_node)
+            self.add_output_graph_node(max_enter_node)
 
-        return qint8_const_node.name, min_node.name, max_node.name
+            return qint8_const_enter_node.name, min_enter_node.name, max_enter_node.name
+        else:
+            self.add_output_graph_node(qint8_const_node)
+            self.add_output_graph_node(min_node)
+            self.add_output_graph_node(max_node)
+
+            return qint8_const_node.name, min_node.name, max_node.name
