@@ -18,13 +18,14 @@
 from tensorflow.core.framework import graph_pb2
 from tensorflow.python.platform import gfile
 from neural_compressor.utils.utility import dump_elapsed_time
+from neural_compressor.adaptor.tf_utils.quantize_graph_common import QuantizeGraphHelper
 
 from .quantize_graph_base import QuantizeGraphBase
-from .quantize_graph_common import QuantizeGraphHelper
 from .quantize_graph_conv import FuseNodeStartWithConv2d
 from .quantize_graph_concatv2 import FuseNodeStartWithConcatV2
 from .quantize_graph_matmul import FuseNodeStartWithMatmul
 from .quantize_graph_pooling import FuseNodeStartWithPooling
+from .quantize_graph_bn import FuseNodeStartWithFusedBatchNormV3
 
 class QuantizeGraphForIntel(QuantizeGraphBase):
     """
@@ -32,7 +33,7 @@ class QuantizeGraphForIntel(QuantizeGraphBase):
     """
 
     def __init__(self, input_graph, input_node_names, output_node_names, op_wise_config, op_wise_sequences, device, \
-                 fake_quant=False):
+                 fake_quant=False, new_api=False, performance_only=False):
         """Quantize Graph For Intel Cpu
 
         Arguments:
@@ -61,10 +62,16 @@ class QuantizeGraphForIntel(QuantizeGraphBase):
 
         self.device = device
         self.fake_quant = fake_quant
+        self.new_api = new_api
+        self.performance_only = performance_only
 
         self.all_quantizable_node = []
+        self.exclude_node_names = []
         self.register_transformer("MaxPool", FuseNodeStartWithPooling)
+        self.register_transformer("MaxPool3D", FuseNodeStartWithPooling)
         self.register_transformer("Conv2D", FuseNodeStartWithConv2d)
+        self.register_transformer("Conv3D", FuseNodeStartWithConv2d)
+        self.register_transformer("FusedBatchNormV3", FuseNodeStartWithFusedBatchNormV3)
         self.register_transformer("DepthwiseConv2dNative", FuseNodeStartWithConv2d)
         self.register_transformer("AvgPool", FuseNodeStartWithPooling)
         self.register_transformer("ConcatV2", FuseNodeStartWithConcatV2)
@@ -74,6 +81,7 @@ class QuantizeGraphForIntel(QuantizeGraphBase):
     def do_transform(self):
         count = 0
         remove_redundant_quant_flag = False
+        op_wise_config_name_list = list(self.op_wise_config.keys())
         all_node_length = len(self.op_wise_config)
         for _, node in enumerate(self.input_graph.node):
             if node in self.input_graph.node and node.op in self.transformers \
@@ -81,18 +89,22 @@ class QuantizeGraphForIntel(QuantizeGraphBase):
                 count += 1
                 if count == all_node_length:
                     remove_redundant_quant_flag = True
-
-                self.input_graph, quantizable_node_names = self.transformers[node.op](
+                self.input_graph, quantizable_node_names, exclude_node_names= self.transformers[node.op](
                     input_graph=self.input_graph,
                     patterns=self.op_wise_seq[node.op],
                     remove_redundant_quant_flag=remove_redundant_quant_flag,
                     op_wise_cfg=self.op_wise_config[node.name],
+                    op_wise_config_name_list=op_wise_config_name_list,
                     start_node_name=node.name, device=self.device, \
-                    fake_quant=self.fake_quant).apply_the_transform()
+                    fake_quant=self.fake_quant, new_api=self.new_api,
+                    performance_only=self.performance_only).apply_the_transform()
                 if quantizable_node_names:
-                    if node.op in ('ConcatV2', 'MaxPool', 'AvgPool'):
+                    if node.op in ('ConcatV2', 'MaxPool', 'MaxPool3D', 'AvgPool'):
                         self.all_quantizable_node.extend([[i] for i in quantizable_node_names])
                     else:
                         self.all_quantizable_node.append(quantizable_node_names)
+                if exclude_node_names:
+                    self.exclude_node_names.extend(exclude_node_names)
+
         return self.remove_dead_nodes(self.input_graph, self.output_node_names), \
-            self.all_quantizable_node
+            self.all_quantizable_node, self.exclude_node_names
