@@ -19,7 +19,8 @@ from neural_compressor.utils.utility import LazyImport
 from .filter import Filter, filter_registry
 tf = LazyImport('tensorflow')
 
-@filter_registry(filter_type="LabelBalanceCOCORecord", framework="tensorflow")
+@filter_registry(filter_type="LabelBalanceCOCORecord", framework="tensorflow, \
+                 inteltensorflow, tensorflow_itex")
 class LabelBalanceCOCORecordFilter(Filter):
     def __init__(self, size=1):
         self.size = size
@@ -28,7 +29,8 @@ class LabelBalanceCOCORecordFilter(Filter):
         return tf.math.equal(len(label[0]), self.size)
 
 @filter_registry(filter_type="LabelBalanceCOCORaw", framework="tensorflow, \
-    pytorch, mxnet, onnxrt_qlinearops, onnxrt_integerops, onnxrt_qdqops")
+                 inteltensorflow, tensorflow_itex, \
+                 pytorch, mxnet, onnxrt_qlinearops, onnxrt_integerops")
 class LabelBalanceCOCORawFilter(Filter):
     def __init__(self, size=1):
         self.size = size
