@@ -69,6 +69,9 @@ class Quantization(Component):
             if self._eval_dataloader is None:
                 eval_dataloader_cfg = deep_get(cfg, 'evaluation.accuracy.dataloader')
                 if eval_dataloader_cfg is None:
+                    logger.info("Because both eval_dataloader_cfg and user-defined eval_func are None," \
+                        " automatically setting 'tuning.exit_policy.performance_only = True'.")
+                    deep_set(cfg, 'tuning.exit_policy.performance_only', True)
                     logger.info("Generate a fake evaluation function.")
                     self._eval_func = self._fake_eval_func
                 else:
@@ -78,14 +81,17 @@ class Quantization(Component):
 
                     self._eval_dataloader = create_dataloader(self.framework, \
                                                               eval_dataloader_cfg)
+        if os.environ.get("PERFORMANCE_ONLY") in ['0', '1']:
+            performance_only = bool(int(os.environ.get("PERFORMANCE_ONLY")))
+            deep_set(cfg, 'tuning.exit_policy.performance_only', performance_only)
+            logger.info("Get environ 'PERFORMANCE_ONLY={}'," \
+                " force setting 'tuning.exit_policy.performance_only = True'.".format(performance_only))
 
     def _create_calib_dataloader(self, cfg):
         approach_cfg = deep_get(cfg, 'quantization.approach')
         if self._train_func:
             assert approach_cfg == 'quant_aware_training', 'q_func property should not ' \
                    'set for {}'.format(approach_cfg)
-            assert self._calib_dataloader is None, 'q_func has provided by user, ' \
-                   'calib_dataloader property should not be set.'
 
         if self._calib_dataloader is None and self._train_func is None:
             if approach_cfg == 'post_training_static_quant':
