@@ -144,13 +144,22 @@ class TuneStrategy(object):
 
         self.mixed_precision_mode = bool('mixed_precision' in self.cfg) or \
             bool('graph_optimization' in self.cfg)
-
+        if self.mixed_precision_mode:
+            self.dtype = copy.deepcopy(self.cfg.graph_optimization.precisions) if \
+                self.cfg.graph_optimization else \
+                copy.deepcopy(self.cfg.mixed_precision.precisions)
+            if 'fp32' in self.dtype and len(self.dtype) > 1:
+                self.dtype.remove('fp32')
+        else:
+            self.dtype = ['int8']
+ 
         if 'tensorflow' in framework:
             framework_specific_info.update(
                 {"inputs": self.cfg.model.inputs,
                  "outputs": self.cfg.model.outputs,
                  'workspace_path': self.cfg.tuning.workspace.path,
-                 'recipes': self.cfg.quantization.recipes})
+                 'recipes': self.cfg.quantization.recipes,
+                 'performance_only': self.cfg.tuning.exit_policy.performance_only})
         if framework == 'mxnet':
             framework_specific_info.update({"q_dataloader": q_dataloader})
         if 'onnxrt' in framework.lower():
@@ -344,6 +353,7 @@ class TuneStrategy(object):
             logger.info("Neither evaluation function nor metric is defined." \
                         " Generate a quantized model with default quantization configuration.")
             self.cfg.tuning.exit_policy.performance_only = True
+            logger.info("Force setting 'tuning.exit_policy.performance_only = True'.")
             logger.info("Generate a fake evaluation function.")
             self.eval_func = self._fake_eval_func
 
@@ -631,13 +641,12 @@ class TuneStrategy(object):
 
                 last_tune = [weighted_acc]
 
-            last_tune_msg = '[Accuracy (int8|fp32):' + \
+            last_tune_msg = '[Accuracy ({}|fp32):'.format(' '.join(self.dtype)) + \
                 ''.join([' {:.4f}|{:.4f}'.format(last, base) for last, base in \
                 zip(last_tune, self.tune_data['baseline'])]) + \
-                ''.join([', {} (int8|fp32): {:.4f}|{:.4f}'.format( \
-                x, y, z) for x, y, z in zip( \
-                self.objectives.representation, self.last_tune_result[1], self.baseline[1]) \
-                if x != 'Accuracy']) + ']'
+                ''.join([', {} ({}|fp32): {:.4f}|{:.4f}'.format(w, x, y, z) for \
+                w, x, y, z in zip(self.objectives.representation, self.dtype, \
+                self.last_tune_result[1], self.baseline[1]) if x != 'Accuracy']) + ']'
         else: # pragma: no cover
             last_tune_msg = 'n/a'
             for name in self.tune_data.keys() - {'baseline'}:
@@ -737,7 +746,7 @@ class TuneStrategy(object):
             # some fields in tuning section of yaml, such as tensorboard, snapshot, resume.
             if self._same_yaml(tuning_history['cfg'], self.cfg):
                 for history in tuning_history['history']:
-                    if history and history['tune_cfg'] == tune_cfg:
+                    if history and history['tune_cfg']['op'] == tune_cfg['op']:
                         return tuning_history
 
         return None
@@ -845,4 +854,4 @@ class TuneStrategy(object):
     
 
 
-        
\ No newline at end of file
+        
